
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gEKG.h"


// ----------------------------------------------------------------------
//
// Konstruktor
//
// ----------------------------------------------------------------------

gEKG::gEKG( const char * ttyName ) : Gtk::Window(GTK_WINDOW_TOPLEVEL), 
	       MainBox( false, 2 ),
               UpperBox( false, 4 ), 
               StartButton( "Run " ), 
               PrintButton( "Print " ), 
               FilterButton( "Filter " ), 
	       HelpButton( "Help " ),
	       LabelBox( true, 0 ),
	       Label_I( "I" ), Label_II( "II" ), Label_III( "III" ),
	       Label_aVR( "aVR" ), Label_aVL( "aVL" ), Label_aVF( "aVF" ),
               // Papier 265 mm, min 5 mm/s
	       Buffer( static_cast<int>( 265 / 5 / EKG_TimeStep ) )
{ 
  // Signalflu
  Source.connectTo( & hFilter );
  hFilter.connectTo( & lFilter );
  lFilter.connectTo( & Freq );
  Freq.connectTo( & Display );
  Display.connectTo( & Buffer );
  lFilter.set_enable( false );
  hFilter.set_enable( false );
  Freq.setBeep( true );
  Freq.setFreq( false );
  Source.serialOpen( ttyName );
  // PrintDialog
  PrintDialog.set_modal( true );
  PrintDialog.get_ok_button()->clicked.connect( 
      slot( this, &gEKG::printDialogCallback ) );
  // FilterDialog
  FilterDialog.OKButton.clicked.connect( 
      slot( this, &gEKG::filterApplyCallback ) );
  FilterDialog.AButton.clicked.connect( 
      slot( this, &gEKG::filterApplyCallback ) );
  // Events verbinden
  StartButton.clicked.connect( slot( this, &gEKG::startCallback ) );
  PrintButton.clicked.connect( slot( this, &gEKG::printCallback ) );
  FilterButton.clicked.connect( slot( this, &gEKG::filterCallback ) );
  HelpButton.clicked.connect( slot( this, &gEKG::helpCallback ) );
  destroy.connect( slot( this, &gEKG::destroy_handler ) );
  key_press_event.connect( slot( this, &gEKG::keyCallback ) );
  // X Einstellungen
  set_wmclass( "EKG", "EKG" );
  // Aussehen
  set_border_width( 5 );
  DisplayFrame.set_shadow_type( GTK_SHADOW_IN );
  add( MainBox );
  MainBox.pack_start( UpperBox, false, false, 0 );
  MainBox.pack_start( DisplayFrame, true, true, 0 );
  UpperBox.pack_start( StartButton, false, false, 0 );
  UpperBox.pack_start( PrintButton, false, false, 0 );
  UpperBox.pack_start( FilterButton, false, false, 0 );
  UpperBox.pack_start( HelpButton, false, false, 0 );
  UpperBox.pack_end( Freq, false, false, 0 );
  DisplayFrame.add( FrameBox );
  FrameBox.pack_start( LabelBox, false, false, 5 );
  FrameBox.pack_start( Display, true, true, 0 );
  ebox_I.add( Label_I );
  ebox_II.add( Label_II );
  ebox_III.add( Label_III );
  ebox_aVR.add( Label_aVR );
  ebox_aVL.add( Label_aVL );
  ebox_aVF.add( Label_aVF );
  LabelBox.pack_start( ebox_I, true, true, 0 );
  LabelBox.pack_start( ebox_II, true, true, 0 );
  LabelBox.pack_start( ebox_III, true, true, 0 );
  LabelBox.pack_start( ebox_aVR, true, true, 0 );
  LabelBox.pack_start( ebox_aVL, true, true, 0 );
  LabelBox.pack_start( ebox_aVF, true, true, 0 );
  // Tooltips
  Tips.set_tip( Freq, "Pulse" );
  show_all();
}


// ----------------------------------------------------------------------
//
// Event Handler
// 
// ----------------------------------------------------------------------

/* Tastendruck */
gint gEKG::keyCallback( GdkEventKey * e )
{
  switch ( e->keyval )
  {
    case 0x72 :   // r
      StartButton.clicked();
      return true;
    case 0x68 :   // h
    case 0xffbe : // F1
      HelpButton.clicked();
      return true;
    case 0x66 :   // f
      FilterButton.clicked();
      return true;
    case 0x71 :   // q
      destroy();
      return true;
    case 0x70 :   // p
    case 0xff61 : // Print
      PrintButton.clicked();
      return true;
   }
   return false;
}


/* Start Button */
void gEKG::startCallback()
{
  if ( StartButton.get_active() )
  {
    Buffer.reset();
    Display.reset();
    Source.start();
  }
  else
  {
    Source.stop();
    Freq.reset();
  }
}


/* Print Button */
void gEKG::printCallback()
{
  PrintDialog.show();
}


/* Hier wird gedruckt. Aufruf vom pintDialog OK */
void gEKG::printDialogCallback()
{
  EKGplotter plotter( Buffer );
  plotter.ts = PrintDialog.ts;
  plotter.vs = PrintDialog.vs;
  plotter.name = PrintDialog.pname;
  if ( ! PrintDialog.pdate.empty() )
    plotter.name += ", " + PrintDialog.pdate;
  /* Druck in Datei */
  if ( PrintDialog.toFile ) 
  {
    ofstream out( PrintDialog.fname.c_str() );
    plotter.toPostscript( out );
    out.close();
  }
  /* Druck in Datei */
  else 
  {
    int iopipe[2];
    pid_t newpid;

    pipe( iopipe );
    if ( ( newpid = fork() ) < 0 ) 
      throw string("gEKG::printDialogCallback : Can't fork.");
    if ( newpid == 0 ) // Kind, wird zu lpr
    {
      if ( PrintDialog.fname.empty() )
        PrintDialog.fname = "lpr";
      dup2( iopipe[0], STDIN_FILENO );
      close( iopipe[0] ); 
      close( iopipe[1] );
      execl( "/bin/sh", "sh", "-c", PrintDialog.fname.c_str(), NULL );
    }
    else // Vater, Daten schicken
    {
      // Wenn das Kind (lpr) tot ist, ist iopipe[0] geschlossen, und
      // Schreibzugriffe auf ioipipe[1] schlagen fehl. Das auftretende
      // Signal SIGPIPE wird ignoriert (SIG_IGN) und wir hoffen, da
      // ostream mit den Fehlern umgehen kann und nicht auf das
      // Standardverhalten (SIG_DFL) umgeschaltet wird [man 2 signal].
      signal( SIGPIPE, SIG_IGN );
      close( iopipe[0] );
      ofstream out( iopipe[1] );
      plotter.toPostscript( out );
      close( iopipe[1] );
    }
  }
}



// Filter Button
void gEKG::filterCallback()
{
  FilterDialog.setValues( 
      lFilter.get_enable(), lFilter.getLFreq(),
      hFilter.get_enable(), hFilter.getHTimeConstant() );
  FilterDialog.show();
}


// Filterwerte bernehmen
void gEKG::filterApplyCallback()
{
  lFilter.set_enable( FilterDialog.getLow() );
  lFilter.setLFreq( FilterDialog.getLowVal() );
  hFilter.set_enable( FilterDialog.getHigh() );
  hFilter.setHTimeConstant( FilterDialog.getHighVal() );
}



/* Hauptfenster Destroy */
void gEKG::destroy_handler()
{
  Gtk::Main::quit();
}


/* Hauptfenster Delete */
int gEKG::delete_event_impl(GdkEventAny *event)
{
  Source.stop();
  return false; 
}


// ----------------------------------------------------------------------
//
// Hauptprogramm
// 
// ----------------------------------------------------------------------

/*
 * Hauptprogramm. Aufruf: gEKG /dev/ttySx
 */
int main (int argc, char *argv[])
{
  try
  {
    Gtk::Main kit(argc, argv);
    if ( argc != 2 ) throw string("Synopsis: gekg /dev/ttySx");
    gEKG w( argv[1] );
    kit.run();
    return 0;
  }
  catch (string s) { cerr << s << endl; }
}


